@@ -451,14 +451,7 @@ impl<'a, 'tcx, F: Fn(Ty<'tcx>) -> bool> MoveDataBuilder<'a, 'tcx, F> {
             Rvalue::Ref(..)
             | Rvalue::RawPtr(..)
             | Rvalue::Discriminant(..)
-            | Rvalue::NullaryOp(
-                NullOp::SizeOf
-                | NullOp::AlignOf
-                | NullOp::OffsetOf(..)
-                | NullOp::UbChecks
-                | NullOp::ContractChecks,
-                _,
-            ) => {}
+            | Rvalue::NullaryOp(NullOp::RuntimeChecks(_)) => {}
         }
     }
 
