@@ -1,7 +1,7 @@
 //! The `wasm32-wali-linux-musl` target is a wasm32 target compliant with the
 //! [WebAssembly Linux Interface](https://github.com/arjunr2/WALI).
 
-use crate::spec::{Cc, LinkerFlavor, Target, TargetMetadata, base};
+use crate::spec::{Arch, Cc, LinkerFlavor, Target, TargetMetadata, base};
 
 pub(crate) fn target() -> Target {
     let mut options = base::linux_wasm::opts();
@@ -30,7 +30,7 @@ pub(crate) fn target() -> Target {
         },
         pointer_width: 32,
         data_layout: "e-m:e-p:32:32-p10:8:8-p20:8:8-i64:64-i128:128-n32:64-S128-ni:1:10:20".into(),
-        arch: "wasm32".into(),
+        arch: Arch::Wasm32,
         options,
     }
 }
